#include <Adafruit_GFX.h>
#include "Adafruit_ThinkInk.h"
#include "Adafruit_seesaw.h"
#include <seesaw_neopixel.h>
#include <Preferences.h>
#include <Adafruit_NeoPixel.h>

//put the fonts in a separate file.
#include "font.h"



//Storage Setup////////////////////////////////
Preferences preferences;
int timeCount = millis();
bool updateFlag = false;
//////////////////////////////////////////////

//Setup State Machine ////////////////////////
enum class setupMode : uint8_t {
  Regular,
  Set_year,
  Set_month,
  Set_ver
};
setupMode calMode = setupMode::Regular;
//////////////////////////////////////////////

//Eink Setup///////////////////////////////////
#define EPD_DC 14
#define EPD_CS 32
#define EPD_BUSY 15  // can set to -1 to not use a pin (will wait a fixed delay)
#define SRAM_CS 12
#define EPD_RESET 33  // can set to -1 and share with microcontroller Reset!
#define EPD_SPI &SPI  // primary SPI

ThinkInk_154_Mono_D67 display(EPD_DC, EPD_RESET, EPD_CS, SRAM_CS, EPD_BUSY, EPD_SPI);
//////////////////////////////////////////////

//Neopixels Setup/////////////////////////////
#define NUMPIXELS 72
Adafruit_NeoPixel pixels(NUMPIXELS, 13, NEO_RGB + NEO_KHZ800);
///////////////////////////////////////////////



//Encoder Setup ////////////////////////////////////
#define SS_SWITCH 24
#define SS_NEOPIX 6

#define SEESAW_ADDR 0x36

Adafruit_seesaw ss;
seesaw_NeoPixel sspixel = seesaw_NeoPixel(1, SS_NEOPIX, NEO_GRB + NEO_KHZ800);

int32_t encoder_position;
//////////////////////////////////////////////

//Calendar Tracker Variables///////////////////////
//Nuitari globals
uint8_t nuiPosn = 2;
bool nuiAlignment = false;
const uint8_t nuiMax = 8;
char* nuiPhase[nuiMax] = { "New", "Waxing Crescent", "First Quarter", "Waxing Gibbous", "Full", "Waning Gibbous", "Last Quarter", "Waning Crescent" };
char* nuiSanction[nuiMax] = { "Low", "Waxing", "Waxing", "High", "High", "Waning", "Waning", "Low" };
char* nuiPhaseLetters[nuiMax] = { "z", "d", "g", "j", "Z", "p", "r", "w" };
int nuiLights[nuiMax] = { 1, 0, 2, 3, 4, 5, 6, 7 };
//Lunitari globals
uint8_t luniPosn = 0;
bool luniAlignment = false;
const uint8_t luniMax = 28;
char* luniPhase[luniMax] = { "New",
                             "Waxing Crescent", "Waxing Crescent", "Waxing Crescent", "Waxing Crescent", "Waxing Crescent", "Waxing Crescent",
                             "First Quarter",
                             "Waxing Gibbous", "Waxing Gibbous", "Waxing Gibbous", "Waxing Gibbous", "Waxing Gibbous", "Waxing Gibbous",
                             "Full",
                             "Waning Gibbous", "Waning Gibbous", "Waning Gibbous", "Waning Gibbous", "Waning Gibbous", "Waning Gibbous",
                             "Last Quarter",
                             "Waning Crescent", "Waning Crescent", "Waning Crescent", "Waning Crescent", "Waning Crescent", "Waning Crescent" };
char* luniPhaseLetters[luniMax] = { "z",
                                    "a", "b", "c", "d", "e", "f",
                                    "g",
                                    "h", "i", "i", "j", "k", "l",
                                    "Z",
                                    "m", "n", "p", "q", "r", "s"
                                                             "t",
                                    "u", "v", "v", "w", "x", "y" };


char* luniSanction[luniMax] = { "Low", "Low", "Low", "Low",
                                "Waxing", "Waxing", "Waxing", "Waxing", "Waxing", "Waxing", "Waxing",
                                "High", "High", "High", "High", "High", "High", "High",
                                "Waning", "Waning", "Waning", "Waning", "Waning", "Waning", "Waning",
                                "Low", "Low", "Low" };
int luniLights[luniMax] = { -1, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34 };
//Solinari global Variable
uint8_t soliStart[3] = { 16, 28, 4 };
uint8_t soliPosn = 0;
const uint8_t soliMax = 36;
bool soliAlignment = false;
char* soliPhase[soliMax] = { "New",
                             "Waxing Crescent", "Waxing Crescent", "Waxing Crescent", "Waxing Crescent", "Waxing Crescent", "Waxing Crescent", "Waxing Crescent", "Waxing Crescent",
                             "First Quarter",
                             "Waxing Gibbous", "Waxing Gibbous", "Waxing Gibbous", "Waxing Gibbous", "Waxing Gibbous", "Waxing Gibbous", "Waxing Gibbous", "Waxing Gibbous",
                             "Full",
                             "Waning Gibbous", "Waning Gibbous", "Waning Gibbous", "Waning Gibbous", "Waning Gibbous", "Waning Gibbous", "Waning Gibbous", "Waning Gibbous",
                             "Last Quarter",
                             "Waning Crescent", "Waning Crescent", "Waning Crescent", "Waning Crescent", "Waning Crescent", "Waning Crescent", "Waning Crescent", "Waning Crescent" };
char* soliPhaseLetters[soliMax] = { "z",
                                    "a", "b", "b", "c", "d", "d", "e", "f",
                                    "g",
                                    "h", "i", "i", "j", "j", "k", "k", "l",
                                    "Z",
                                    "m", "n", "o", "p", "q", "q", "r", "s"
                                                                       "t",
                                    "u", "v", "v", "w", "w", "x", "x", "y" };

char* soliSanction[soliMax] = { "Low", "Low", "Low", "Low", "Low",
                                "Waxing", "Waxing", "Waxing", "Waxing", "Waxing", "Waxing", "Waxing", "Waxing", "Waxing",
                                "High", "High", "High", "High", "High", "High", "High", "High", "High",
                                "Waning", "Waning", "Waning", "Waning", "Waning", "Waning", "Waning", "Waning", "Waning",
                                "Low", "Low", "Low", "Low" };
int soliLights[soliMax] = { -1, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 57, 56, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 35 };
//Day and Year Global Variables
int16_t year = 353;
#define initMem 5
#define yearHigh 0
#define yearLow 1
uint8_t yearPattern = 0;
int16_t day = 28 * 3;
#define dayHigh 2
#define dayLow 3
int dayMax = 335;

uint8_t month = 4;
uint8_t dayOfMonth = 1;
uint8_t dayOfWeek = 1;

//Set for Aurium Empire, Feel free to change to your calendar. 
//Saturday, Sunday, Monday, Tuesday, Wednesday Thursday, Friday

int whichCal = 0;
const char* calVer[11] PROGMEM = { "Aurim", "Dwarven", "Elven", "Ergothian", "Goblin", "Kender", "Plainsfolk", "Solamnic", "English", "League", "Tamire" };

const char* days[11][7] PROGMEM = {
  { "Aurus", "Solus", "Vendus", "Holus", "Lunus", "Angonus", "Tellus" },                                //Aurim
  { "Agorin", "Brenzik", "Mithrik", "Adamachtis", "Aurachil", "Cuprig", "Ferramis" },                   //Dwarven
  { "World Tree", "Gateway", "Bright Eye", "Night's Eye", "Dead Eye", "Dream Dance", "Winged Trade" },  //Elven
  { "Boreadai", "Gileadai", "Luindai", "Nuindai", "Soldai", "Manthus", "Shinarai" },                    //Ergothian
  { "Flank", "Guard", "Pain", "Fear", "Rout", "	Strife", "Feint" },                                     //Goblin
  { "Shadow Day", "Praise Day", "Light Day", "Fire Day", "Winds Day", "Waters Day", "Ground Day" },     //Kender
  { "Council Day", "Friend Day", "Hunt Day", "Share Day", "Gather Day", "Barter Day", "Worship Day" },  //Plainsfolk
  { "Bracha", "Linaras", "Palast", "Majetag", "Kirinor", "Misham", "Bakukal" },                         //Solamnic
  { "Saturday", "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday" },                     //English
  { "Sargadan", "Ambedan", "Praidan", "Kristodan", "Morgadan", "Nuradan", "Eredan" },                   //League
  { "World Tree", "Gateway", "Bright Eye", "Night's Eye", "Dead Eye", "Dream Dance", "Winged Trade" }   //Tamire
};
const char* months[11][13] PROGMEM = {
  { "", "Neraran", "Eldaran", "Feloran", "Kathoran", "Gratharan", "Daiduran", "Kaldoran", "Randoran", "Amodan", "Baladan", "Zendoran", "Brectadan" },                                                      //Aurim
  { "", "Dark-Crypt", "Dark-Deep", "Damp-Mood", "Damp-Chisel", "Dry-Anvil", "Dry-Axe", "Dry-Heat", "Dry-Forge", "Cold-Hammer", "Cold-Steel", "Cold-Rust", "Cold-Lode" },                                   //Dwarven
  { "", "Winter Night", "Winter Deep", "Spring Dawning", "Spring Rain", "Spring Blossom", "Summer Home", "Summer Run", "Summer End", "Autumn Harvest", "Autumn Twilight", "Autumn Dark", "Winter Come" },  //Elven
  { "", "Aelmont", "Rannmont", "Mishamont	", "Chislmont", "Bran", "Corij", "Argonn", "Sirrimont", "Reorxmont", "Hiddumont", "H\’rarmont", "Phoenix" },                                                     //Ergothian
  { "", "Famine", "Madness", "Girding", "Scouting", "Campaign", "Siege", "Flame", "Sack", "Loot", "Betrayal", "Pestilence", "Reward" },                                                                    //Goblin
  { "", "Snowfun", "Darktime", "Windsong", "Raindrum", "Flowerfield", "Homefriends", "Wandertime", "Summerlaze", "Harvestfete", "Leafplay", "Bleakcold", "Blessings" },                                    //Kender
  { "", "Ice Glaze", "Snow Deep", "Mountain Thaw", "Earth Wakes", "Flower Blooms", "Home Earth", "Raging Fire", "Dying Ember", "Harvest Home", "Leaf Gild", "Dark Cold", "Frost Eve" },                    //Plainsfolk
  { "", "Newkolt", "Deepkolt", "Brookgreen", "Yurthgreen", "Fleurgreen", "Holmswelth", "Fierswelt", "Paleswelt", "Reapember", "Gildember", "Darkember", "Frostkolt" },                                     //Solamnic
  { "", "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" },                                                                        //English
  { "", "Octander", "Novander", "Decander", "Stellus", "Dodecander", "Ambeoutus", "Duander", "Treander", "Eregatus", "Pentander", "Skythus", "Septander" },                                                //League
  { "", "Tiger–Ice", "Waking–Bear", "Jijin", "Nesting–Hawk", "Running–Foal", "Qu\’uan", "Hunting–Tiger", "Soaring–Wyrm", "Mislaxa", "Sleeping–Owl", "Digging–Hare", "Grey–Fox" }                           //Tamire
};
//////////////////////////////////////////////





void setup() {
  Serial.begin(115200);
  display.begin(THINKINK_MONO);

  //recover saved data
  preferences.begin("cal", false);
  year = preferences.getUInt("year", 353);
  day = preferences.getInt("day", 84);
  whichCal = preferences.getInt("whichCal", 0);
  preferences.end();

  //start connection to encoder
  if (!ss.begin(SEESAW_ADDR) || !sspixel.begin(SEESAW_ADDR)) {
    Serial.println("Couldn't find seesaw on default address");
    while (1) delay(10);
  }
  Serial.println("seesaw started");

  sspixel.setBrightness(20);
  sspixel.show();


  // use a pin for the built in encoder switch
  ss.pinMode(SS_SWITCH, INPUT_PULLUP);

  // get starting position
  encoder_position = ss.getEncoderPosition();

  //set encoder up as an interrupt
  Serial.println("Turning on interrupts");
  delay(10);
  ss.setGPIOInterrupts((uint32_t)1 << SS_SWITCH, 1);
  ss.enableEncoderInterrupt();

  //setup NeoPixels
  pixels.begin();
  pixels.clear();
  pixels.show();

  //Initial update on calendar and screen
  calUpdate();
  drawScreen();
}

void loop() {
  //have the button switch between the states/screens
  if (!ss.digitalRead(SS_SWITCH)) {
    Serial.println("Button pressed!");
    if (calMode == setupMode::Regular) {
      calMode = setupMode::Set_year;
      drawYearScreen();
    } else if (calMode == setupMode::Set_year) {
      calMode = setupMode::Set_month;
      drawMonthScreen();
    } else if (calMode == setupMode::Set_month) {
      calMode = setupMode::Set_ver;
      drawVerScreen();
    } else if (calMode == setupMode::Set_ver) {
      saveDatUpdate();
      calMode = setupMode::Regular;
      drawScreen();
    }
    //debounce the button
    delay(300);
  }

  //handle the encoder
  int32_t new_position = ss.getEncoderPosition();


  if (encoder_position != new_position) {
    //if in regular mode,  move the day forward or backward and handle the year if you cross the boundry
    if (calMode == setupMode::Regular) {
      Serial.println(new_position);  // display new position
      if (new_position > encoder_position) {
        day++;
        if (day > dayMax) {
          day = 0;
          year++;
        }
      } else if (new_position < encoder_position) {
        day--;
        if (day < 0) {
          day = dayMax;
          year--;
        }
      }
      calUpdate();
      drawScreen();
      timeCount = millis();
      updateFlag = true;
    }

    //in year mode, add or subtract the year
    else if (calMode == setupMode::Set_year) {
      Serial.println(new_position);  // display new position
      if (new_position > encoder_position) year++;
      else year--;
      Serial.print("Year=");
      Serial.println(year);
      calUpdate();
      drawYearScreen();
    }

    //in month mode, add or subtract a month.
    else if (calMode == setupMode::Set_month) {
      Serial.println(new_position);  // display new position
      if (new_position > encoder_position) {
        day += 28;
        if (day > dayMax) day = dayMax - 1;
      } else if (new_position < encoder_position) {
        day -= 28;
        if (day < 0) day = 0;
      }
      calUpdate();
      drawMonthScreen();
    }
    //in calender version mode encoder will change the language
    else if (calMode == setupMode::Set_ver) {
      Serial.println(new_position);  // display new position
      if (new_position > encoder_position) {
        whichCal++;
        if (whichCal > 10) whichCal = 0;
        //drawScreen();
      } else if (new_position < encoder_position) {
        whichCal--;
        if (whichCal < 0) whichCal = 10;
        //drawScreen();
      }
      calUpdate();
      drawVerScreen();
    }

    encoder_position = new_position;  // and save for next round
  }

  //if the day hasn't been changed in three seconds, update the eeprom.
  if (timeCount + 30000 <= millis() && updateFlag) {
    saveDatUpdate();
    updateFlag = false;
  }
}

void calUpdate() {
  //find the year pattern there is three year cycle
  yearPattern = year % 3;

  //use the day variable to find the calendar details
  month = (day / 28) + 1;
  dayOfMonth = (day % 28) + 1;
  dayOfWeek = (dayOfMonth % 7);

  //update the moon cycles
  nuiPosn = (day + 2) % nuiMax;
  luniPosn = day % luniMax;
  soliPosn = (day + soliStart[yearPattern]) % soliMax;

  //check for alignments
  nuiAlignment = false;
  luniAlignment = false;
  soliAlignment = false;

  if ((nuiSanction[nuiPosn] == soliSanction[soliPosn])) {
    nuiAlignment = true;
    soliAlignment = true;
  }
  if ((nuiSanction[nuiPosn] == luniSanction[luniPosn])) {
    nuiAlignment = true;
    luniAlignment = true;
  }
  if ((soliSanction[soliPosn] == luniSanction[luniPosn])) {
    soliAlignment = true;
    luniAlignment = true;
  }
  moonlightUpdate();
}

void moonlightUpdate() {
  //update the LEDs for the moons
  pixels.clear();  // Set all pixel colors to 'off'

  pixels.setPixelColor(nuiLights[nuiPosn], pixels.Color(97, 52, 235));
  pixels.setPixelColor(luniLights[luniPosn], pixels.Color(255, 0, 0));
  pixels.setPixelColor(soliLights[soliPosn], pixels.Color(200, 200, 200));
  
  pixels.show();
}


//handle the display for each of the modes
void drawYearScreen() {
  display.clearBuffer();
  display.setTextSize(1);
  display.setTextColor(EPD_BLACK);
  display.setCursor(0, 60);
  display.setFont(&Dyer_Arts_and_Crafts14pt7b);
  display.print("Set Year");
  display.setFont(&Dyer_Arts_and_Crafts36pt7b);
  display.setCursor(0, 120);
  if (year > 0) {
    display.print(year);
    display.print(" AC");
  } else {
    display.print(abs(year));
    display.print(" PC");
  }
  display.display();
}

void drawMonthScreen() {
  display.clearBuffer();
  display.setTextSize(1);
  display.setTextColor(EPD_BLACK);
  display.setCursor(0, 60);
  display.setFont(&Dyer_Arts_and_Crafts14pt7b);
  display.print("Set Month");
  display.setFont(&Dyer_Arts_and_Crafts36pt7b);
  display.setCursor(0, 120);
  display.print(month);
  display.display();
}

void drawVerScreen() {
  display.clearBuffer();
  display.setTextSize(1);
  display.setTextColor(EPD_BLACK);
  display.setCursor(0, 60);
  display.setFont(&Dyer_Arts_and_Crafts14pt7b);
  display.print("Set Calendar");
  display.setFont(&Dyer_Arts_and_Crafts18pt7b);
  display.setCursor(0, 120);
  display.print(calVer[whichCal]);
  display.display();
}

void drawScreen() {
  Serial.println(month);
  Serial.println(months[whichCal][month]);
  Serial.println(day);

  display.clearBuffer();

  display.setTextSize(1);


  display.setTextColor(EPD_BLACK);
  display.setFont(&Dyer_Arts_and_Crafts14pt7b);
  display.setCursor(0, 28);
  //display.print("On this ");
  display.print(days[whichCal][dayOfWeek]);
  //display.print(",");
  display.setFont(&Dyer_Arts_and_Crafts36pt7b);
  display.setCursor(0, 100);
  display.print(dayOfMonth);
  display.setFont(&Dyer_Arts_and_Crafts14pt7b);
  if (dayOfMonth == 11) display.print("th of ");
  else if ((dayOfMonth % 10) == 1) display.print("st of ");
  else if ((dayOfMonth % 10) == 2) display.print("nd of ");
  else if ((dayOfMonth % 10) == 3) display.print("rd of ");
  else display.print("th of ");
  display.setCursor(0, 140);
  display.print(months[whichCal][month]);
  display.setCursor(0, 190);
  //display.print("in the year ");
  if (year > 0) {
    display.print(year);
    display.print(" AC");
  } else {
    display.print(abs(year));
    display.print(" PC");
  }

  display.display();
}


//handles saving out static memory
void saveDatUpdate() {
  preferences.begin("cal", false);
  preferences.putUInt("year", year);
  preferences.putInt("day", day);
  preferences.putInt("whichCal", 0);
  preferences.end();

  Serial.println("Save Complete");
}
